@@ -1,4 +1,4 @@
-/* $Id: er_main.c,v 1.92 2013/02/14 15:52:57 rpalsa Exp $
+/* $Id: er_main.c,v 1.92 2013-02-14 15:52:57 rpalsa Exp $
  *
  *   This file is part of the ESO Common Pipeline Library
  *   Copyright (C) 2001-2009 European Southern Observatory
@@ -15,43 +15,39 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /*
  * $Author: rpalsa $
- * $Date: 2013/02/14 15:52:57 $
+ * $Date: 2013-02-14 15:52:57 $
  * $Revision: 1.92 $
- * $Name: esorex-3_10 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
 #   include <config.h>
 #endif
 
+#include <stdlib.h>
+#include <stdio.h>
+#include <string.h>
+#include <ctype.h>
 #include <unistd.h>
 #include <sys/types.h>
-
-#include <string.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <dlfcn.h>
-
-/*
-#include <fitsio.h>
-*/
-
-#include <dirent.h>
-#include <fcntl.h>
 #include <sys/stat.h>
+#include <pwd.h>
+//#include <dirent.h>
+//#include <dlfcn.h>
+//#include <fcntl.h>
+
+#ifndef S_SPLINT_S
+#  include "ltdl.h"
+#endif
 
 #include <cpl.h>
 #include <cpl_version.h>		/* should be fixed */
 
-# ifndef S_SPLINT_S
-#include "ltdl.h"
-# endif
-
 #include "er_main.h"
 #include "er_help.h"
 #include "er_stringarray.h"
@@ -585,6 +581,257 @@ MyOS = 1;
 
 */
 
+inline static int
+_esorex_config_value_get(FILE *fp, const char *key, char *value, size_t sz)
+{
+
+    int has_key = 0;
+
+    char buffer[MAXSTRLENCONF];
+
+    while (fgets(buffer, MAXSTRLENCONF, fp) != NULL) {
+
+        char *s = buffer;
+
+        while (isspace(*s)) {
+            ++s;
+        }
+
+        if ((s[0] != '\0') && (s[0] != '#') &&
+                (strncmp(s, key, strlen(key)) == 0)) {
+
+            s = strchr(s, '=');
+
+            if (s != NULL) {
+
+                char v[MAXSTRLENCONF];
+
+                int nitems = sscanf(s + 1, "%s", v);
+
+                if (nitems == 1) {
+                    has_key = 1;
+                    strncpy(value, v, sz);
+                    break;
+                }
+                else {
+                    has_key = -1;
+                    break;
+                }
+
+            }
+            else {
+                has_key = -1;
+                break;
+            }
+
+        }
+
+    }
+
+    return has_key;
+}
+
+
+inline static void
+_esorex_config_files_set(int argc, char *argv[],
+                         char **syscfg, char **cfg)
+{
+
+    const char *env_config = getenv("ESOREX_CONFIG");
+
+    int iarg;
+
+    const char *_home   = getenv("HOME");
+    const char *_cfg = NULL;
+
+    char *_path   = NULL;
+
+    struct stat sb;
+
+
+    /*
+     *  File path buffer size. Add an extra character for the directory
+     *  separator, and another one for the string terminator.
+     */
+
+    size_t sz  = strlen(GLOBAL_RC_NAME) + 2;
+    size_t _sz = 0;
+
+
+    if (_home == NULL) {
+
+        struct passwd *pw = getpwuid(getuid());
+        _home = pw->pw_dir;
+
+    }
+
+    _sz = strlen(_home) + strlen(GLOBAL_RC_DIR) + 1;
+    sz += (_sz > strlen(SYSCONFDIR)) ? _sz : strlen(SYSCONFDIR);
+
+    _path = malloc(sz);
+
+
+    /*
+     * Try to find a configuration file in the user's home directory.
+     * If it does not exist, try the system wide as a fallback.
+     */
+
+    snprintf(_path, sz, "%s/%s/%s", _home, GLOBAL_RC_DIR, GLOBAL_RC_NAME);
+
+    if ((stat(_path, &sb) == 0) && S_ISREG(sb.st_mode)) {
+        _cfg = _path;
+    }
+    else {
+        snprintf(_path, sz, "%s/%s", SYSCONFDIR, GLOBAL_RC_NAME);
+
+        if ((stat(_path, &sb) == 0) && S_ISREG(sb.st_mode)) {
+            _cfg = _path;
+        }
+    }
+
+    if (_cfg != NULL) {
+        *syscfg = strdup(_cfg);
+    }
+
+    free(_path);
+
+
+    /*
+     * Check if the user has explicitly specified a configuration file,
+     * either setting an environment variable or by passing a command
+     * line option.
+     */
+
+    _cfg = NULL;
+
+    if (env_config != NULL) {
+        _cfg = env_config;
+    }
+
+    for (iarg = 1; iarg < argc; ++iarg) {
+
+        if ((strlen(argv[iarg]) > 2) &&
+                (strncmp(argv[iarg] + 2, "config", 6) == 0)) {
+
+            const char *s = strchr(argv[iarg] + 2, '=');
+
+            if ((s != NULL) && (*(s + 1) != '\0')) {
+                _cfg = ++s;
+            }
+
+        }
+
+    }
+
+    if (_cfg != NULL) {
+        *cfg = strdup(_cfg);
+    }
+
+    return;
+
+}
+
+
+// Does an advance parsing of the command line, environment, and the
+// configuration file, to determine whether CPL has to be initialized
+// for memory debugging. This needs to be known before any CPL call
+// happens!
+
+// FIXME: This is a hack to workaround the fact that the code does
+//        not do user input collection/verification before the core services
+//        are started.
+
+inline static int
+_esorex_memory_mode_get(int argc, char *argv[],
+                        const char *syscfg, const char *cfg)
+{
+    int iarg;
+    int set_memory_mode = 0;
+
+    const char *env_memcheck = getenv("ESOREX_MEM_CHECK");
+
+
+    if ((syscfg != NULL) && (syscfg[0] != '\0')) {
+
+        const char *key = PACKAGE_RESOURCE ".mem-check";
+
+        char value[MAXSTRLENCONF];
+
+        FILE *fp = fopen(syscfg, "r");
+
+        if (fp != NULL) {
+
+            if (_esorex_config_value_get(fp, key, value, MAXSTRLENCONF) == 1) {
+
+                if ((strncmp(value, "TRUE", 4) == 0) ||
+                        strncmp(value, "true", 4) == 0) {
+                    set_memory_mode = 1;
+                }
+            }
+
+            fclose(fp);
+
+        }
+
+    }
+
+    if ((cfg != NULL) && (cfg[0] != '\0')) {
+
+        const char *key = PACKAGE_RESOURCE ".mem-check";
+
+        char value[MAXSTRLENCONF];
+
+        FILE *fp = fopen(cfg, "r");
+
+        if (fp != NULL) {
+
+            if (_esorex_config_value_get(fp, key, value, MAXSTRLENCONF) == 1) {
+
+                if ((strncmp(value, "TRUE", 4) == 0) ||
+                        strncmp(value, "true", 4) == 0) {
+                    set_memory_mode = 1;
+                }
+            }
+
+            fclose(fp);
+        }
+
+    }
+
+    if ((env_memcheck != NULL) &&
+            ((strncmp(env_memcheck, "TRUE", 4) == 0) ||
+             (strncmp(env_memcheck, "true", 4) == 0))) {
+        set_memory_mode = 1;
+    }
+
+
+    for (iarg = 1; iarg < argc; ++iarg) {
+
+        if ((strlen(argv[iarg]) > 2) &&
+                (strncmp(argv[iarg] + 2, "mem-check", 9) == 0)) {
+
+            if (strstr(argv[iarg], "TRUE") || strstr(argv[iarg], "true")) {
+
+                const char *memory_mode = getenv("CPL_MEMORY_MODE");
+
+                if ((memory_mode == NULL) || (strncmp(memory_mode, "0", 1) == 0)) {
+                    set_memory_mode = 1;
+                }
+
+            }
+            else if (strstr(argv[iarg], "FALSE") || strstr(argv[iarg], "false")) {
+                set_memory_mode = 0;
+            }
+
+        }
+
+    }
+
+    return set_memory_mode;
+
+}
+
+
 /**********************************************************************/
 /**
  * @brief      Top-level function of the EsoRex application.
@@ -596,290 +843,182 @@ MyOS = 1;
 
 int main(int argc, char *argv[])
 {
-const char *cpp, *fn = "EsoRex";
+    const char *cpp, *fn = "EsoRex";
 
-char  plugin_name[FILEMAX+2];
-char  date_str[8];
-char  *conf_file_global=NULL, *conf_file_local=NULL;
- 
-const char *cdescr;
+    char  plugin_name[FILEMAX+2];
+    char  date_str[8];
+    char  *conf_file_global=NULL, *conf_file_local=NULL;
 
-cpl_parameter *p = NULL;
-cpl_parameterlist *caller_parameter_list = NULL;
-cpl_msg_severity msg_level = CPL_MSG_ERROR;
+    const char *cdescr;
 
-er_stringarray_t *set_of_frames_filenames = NULL;
- 
-int e_code = 0;
-int e_code2 = 0;
-int flag_mem_check = 1;
-int  n;
+    cpl_parameter *p = NULL;
+    cpl_parameterlist *caller_parameter_list = NULL;
+    cpl_msg_severity msg_level = CPL_MSG_ERROR;
 
-unsigned  init_flag = CPL_INIT_DEFAULT;
+    er_stringarray_t *set_of_frames_filenames = NULL;
 
-FILE  *rcfp;
+    int e_code = 0;
+    int e_code2 = 0;
+    int flag_mem_check = 1;
+    int  n;
+
+    unsigned  init_flag = CPL_INIT_DEFAULT;
+
+    FILE  *rcfp;
 
 
 
-/* check that we're not somebody else... */
+    /* check that we're not somebody else... */
 
-if (getuid() != geteuid()) 
-   {
-   (void) printf("effective user ID not the same as real user ID => no EsoRex...\n");
-   exit(EXIT_FAILURE);
-   }
-
-/* print version of Esorex, and set date of last modif */
-
-er_print_header ();
-(void) strcpy(date_str,"120311");
-
-
-/* Initialization and setup */
-
-cpl_init (init_flag);                    /* also does cpl_msg_init() stuff */
-
-cpl_msg_set_domain (PACKAGE);
-cpl_msg_set_time_off ();
-cpl_msg_set_threadid_off ();
-cpl_msg_set_domain_on ();
-cpl_msg_set_component_off ();
-
-
-
-/* Main processing */
-
-if (lt_dlinit () != 0)
-   {
-   cpl_msg_error (fn, "Unable to initialise ltdl; aborting program");
-   cpl_end ();                  /* stop subsystems of CPL */
-   e_code = -1;
-   goto clean_up;
-   }
-
-
-caller_parameter_list = cpl_parameterlist_new ();
-set_of_frames_filenames = er_stringarray_new ();
-
-#ifdef USE_LOCAL_ESOREX_RC		/* Specific configuration file */
-n = ((int) strlen(LOCAL_RC_PATH)) +
-    ((int) strlen(GLOBAL_RC_NAME)) + ((int) strlen(GLOBAL_RC_EXTENSION));
-n += 4;
-
-conf_file_local = (char *) cpl_malloc((size_t) n);
-if ( conf_file_local == NULL)
-   {
-   cpl_msg_error (er_func, "Could not allocate %d bytes for conf_file_local",n);
-   e_code = -1;
-   goto clean_up;
-   }
-
-(void) strcpy(conf_file_local,LOCAL_RC_PATH);
-(void) strcat(conf_file_local, "/");
-(void) strcat(conf_file_local, GLOBAL_RC_NAME);
-(void) strcat(conf_file_local, GLOBAL_RC_EXTENSION);
-#endif
-
-#ifdef USE_GLOBAL_ESOREX_RC		/* User configuration file */
-cpp = getenv ("HOME");
-if (cpp == NULL) 
-   {
-   n = 0;
-   cpl_msg_warning (er_func, "Env. variable HOME not set");
-   }
-else
-   {
-   n = (int) strlen(cpp);
-   }
-
-n = n + ((int) strlen(GLOBAL_RC_DIR)) +
-    ((int) strlen(GLOBAL_RC_NAME)) + ((int) strlen(GLOBAL_RC_EXTENSION));
-n += 4;
-
-conf_file_global = (char *) cpl_malloc((size_t) n);
-if ( conf_file_global == NULL)
-   {
-   cpl_msg_error (er_func, "Could not allocate %d bytes for conf_file_global",n);
-   e_code = -1;
-   goto clean_up;
-   }
-
-if (cpp == NULL)
-   {
-   (void) strcpy(conf_file_global, "/");
-   }
-else
-   {
-   (void) strcpy(conf_file_global,cpp);
-   (void) strcat(conf_file_global, "/");
-   }
-(void) strcat(conf_file_global, GLOBAL_RC_DIR);
-(void) strcat(conf_file_global, "/");
-(void) strcat(conf_file_global, GLOBAL_RC_NAME);
-(void) strcat(conf_file_global, GLOBAL_RC_EXTENSION);
-/* 
-printf("USE_GLOBAL_ESOREX_RC - conf_file_global = %s\n\n",conf_file_global); 
-*/
-
-/* check if resource file exists, 
-   use .rc file from --prefix dir in configuration of Esorex if necessary,
-   this is needed for Scisoft  */
-
-rcfp = fopen(conf_file_global, "r");
-if (rcfp != NULL)
-   {
-   fclose(rcfp);
-   }
-else
-   {
-   char save_name[1024], new_name[1024];
-
-   (void) strcpy(save_name,conf_file_global);
-
-   /* The SYSCONFDIR is defind with -DSYSCONFDIR="\"@sysconfdir@\"" in the
-    * Makefile.am, while @sysconfdir@ is substituted partly by configure and
-    * partly by make */
-   (void) strcpy(new_name, SYSCONFDIR"/");	
-   (void) strcat(new_name, GLOBAL_RC_NAME);
-   (void) strcat(new_name, GLOBAL_RC_EXTENSION);
-
-   rcfp = fopen(new_name, "r");
-   if (rcfp == NULL)		/* no .rc file in --prefix dir, */
-      {				/* keep original name for compatibility */
-      (void) strcpy(conf_file_global,save_name);
-      }
-   else
-      {
-      fclose(rcfp);
-
-      cpl_free(conf_file_global);
-      n = (int) strlen(new_name) + 4;
-      conf_file_global = (char *) cpl_malloc((size_t) n);
-      if ( conf_file_global == NULL)
-         {
-         cpl_msg_error (er_func, "Could not allocate %d bytes for conf_file_global",n);
-         e_code = -1;
-         goto clean_up;
-         }
-      (void) strcpy(conf_file_global,new_name);
-      }
-   }
-#endif
-
-
-/* Create list of parameters for EsoRex */
-
-er_init_parameters (caller_parameter_list);
-
-cdescr = cpl_get_description(CPL_DESCRIPTION_DEFAULT);
-
-if ((argc == 2) && ((int)strcmp(argv[1],"--V") == 0)) 
-   {
-   (void) printf("           as of %s \n",date_str);
-   (void) printf("\nLibraries used: %s\n\n",cdescr);
-   (void) printf("using %s as resource file\n",conf_file_global);
-   goto end_of_it;
-   }
-
-
-/* Process caller configuration information */
-
-plugin_name[0] = '\0';
-e_code = params_process_configuration (caller_parameter_list,
-                                       conf_file_global, conf_file_local,
-                                       argc, argv,
-                                       plugin_name, set_of_frames_filenames);
-
-(void) sprintf(trace_msg,"param_process_configuration returned %d",e_code);
-ER_TRACEX (trace_msg)
-
-
-/* Process Plugin */
-
-if (e_code == 0 && plugin_name[0] != '\0')
-{
-    e_code = plugin_process_plugin (caller_parameter_list,
-            plugin_name, set_of_frames_filenames,
-            date_str,argc, argv);
-
-    (void) sprintf(trace_msg,"plugin_process_plugin returned %d",e_code); 
-    ER_TRACEX(trace_msg)
-}
-else if(e_code !=-99999)  /* entering just Esorex yields the library versions */
-{
-    cdescr = cpl_get_description(CPL_DESCRIPTION_DEFAULT);
-    (void) printf("\nLibraries used: %s\n\n",cdescr);
-    goto clean_up;
-}
-
-
-
-
-msg_level = cpl_msg_get_level();	/* test, if we are in debug mode */
-if (msg_level == CPL_MSG_DEBUG) 
-   {						/* if debug level set, */
-   e_code2 = cpl_error_get_code ();		/* check the CPL_error subsystem */
-   if (e_code2 != CPL_ERROR_NONE)
-      {
-      cpl_msg_error (fn, "CPL-error '%s' was set in '%s'",
-                     cpl_error_get_message (), cpl_error_get_where ());
-      if (e_code == 0)
-         {
-         cpl_msg_error (fn, "recipe %s returned status: %d\n",plugin_name,e_code);
-         }
-      }
-   }
-
-p = cpl_parameterlist_find (caller_parameter_list,
-                            PACKAGE_RESOURCE ".unload-plugin");
-
-if (cpl_parameter_get_bool(p) != 0) {
-    if (lt_dlexit () != 0) {
-        cpl_msg_error (fn, "Unable to deinitialize ltdl");
+    if (getuid() != geteuid())
+    {
+        fprintf(stderr, "effective user ID not the same as real user ID => no EsoRex...\n");
+        exit(EXIT_FAILURE);
     }
+
+    /* print version of Esorex */
+
+    er_print_header ();
+
+
+    _esorex_config_files_set(argc, argv, &conf_file_global, &conf_file_local);
+
+    if (_esorex_memory_mode_get(argc, argv,
+                                conf_file_global, conf_file_local) != 0) {
+        setenv("CPL_MEMORY_MODE", "1", 0);
+    }
+
+
+    /* Initialization and setup */
+
+    cpl_init (init_flag);                    /* also does cpl_msg_init() stuff */
+
+    cpl_msg_set_domain (PACKAGE);
+    cpl_msg_set_time_off ();
+    cpl_msg_set_threadid_off ();
+    cpl_msg_set_domain_on ();
+    cpl_msg_set_component_off ();
+
+
+
+    /* Main processing */
+
+    if (lt_dlinit () != 0)
+    {
+        cpl_msg_error (fn, "Unable to initialise ltdl; aborting program");
+        cpl_end ();                  /* stop subsystems of CPL */
+        e_code = -1;
+        goto clean_up;
+    }
+
+
+    /* Create list of parameters for EsoRex */
+
+    caller_parameter_list = cpl_parameterlist_new ();
+    set_of_frames_filenames = er_stringarray_new ();
+
+    er_init_parameters (caller_parameter_list);
+
+
+    /* Process caller configuration information */
+
+    plugin_name[0] = '\0';
+    e_code = params_process_configuration (caller_parameter_list,
+                                           conf_file_global, conf_file_local,
+                                           argc, argv,
+                                           plugin_name, set_of_frames_filenames);
+
+    (void) sprintf(trace_msg,"param_process_configuration returned %d",e_code);
+    ER_TRACEX (trace_msg)
+
+
+    /* Process Plugin */
+
+    if (e_code == 0 && plugin_name[0] != '\0')
+    {
+        e_code = plugin_process_plugin (caller_parameter_list,
+                                        plugin_name, set_of_frames_filenames,
+                                        argc, argv);
+
+        (void) sprintf(trace_msg,"plugin_process_plugin returned %d",e_code);
+        ER_TRACEX(trace_msg)
+    }
+    else if(e_code !=-99999)  /* entering just Esorex yields the library versions */
+    {
+        cdescr = cpl_get_description(CPL_DESCRIPTION_DEFAULT);
+        (void) printf("\nLibraries used: %s\n\n",cdescr);
+        goto clean_up;
+    }
+
+
+
+
+    msg_level = cpl_msg_get_level();	/* test, if we are in debug mode */
+    if (msg_level == CPL_MSG_DEBUG)
+    {						/* if debug level set, */
+        e_code2 = cpl_error_get_code ();		/* check the CPL_error subsystem */
+        if (e_code2 != CPL_ERROR_NONE)
+        {
+            cpl_msg_error (fn, "CPL-error '%s' was set in '%s'",
+                           cpl_error_get_message (), cpl_error_get_where ());
+            if (e_code == 0)
+            {
+                cpl_msg_error (fn, "recipe %s returned status: %d\n",plugin_name,e_code);
+            }
+        }
+    }
+
+    p = cpl_parameterlist_find (caller_parameter_list,
+                                PACKAGE_RESOURCE ".unload-plugin");
+
+    if (cpl_parameter_get_bool(p) != 0) {
+        if (lt_dlexit () != 0) {
+            cpl_msg_error (fn, "Unable to deinitialize ltdl");
+        }
+    }
+    else {
+        cpl_msg_warning(fn, "unloading of plugins is inhibited by program "
+                        "option 'unload-plugin'!");
+    }
+
+    p = cpl_parameterlist_find (caller_parameter_list, PACKAGE_RESOURCE ".mem-check");
+    flag_mem_check = cpl_parameter_get_bool (p);
+
+
+    /* cleanup allocated structures and memory */
+
+    clean_up:
+    if (caller_parameter_list) {
+        cpl_parameterlist_delete (caller_parameter_list);
+    }
+
+    if (set_of_frames_filenames) {
+        er_stringarray_delete (set_of_frames_filenames);
+    }
+
+    if (conf_file_local != NULL)
+    {
+        free(conf_file_local);
+    }
+    if (conf_file_global != NULL)
+    {
+        free(conf_file_global);
+    }
+
+    (void) er_help_free();
+    (void) er_manage_sources(3, "", NULL);	/* free buffer of sources */
+
+
+    if (flag_mem_check != 0)	/* see if we should check the memory */
+    {
+        cpl_memory_dump();
+    }
+
+    end_of_it:
+    cpl_end ();			/* stop subsystems of CPL */
+
+    if (e_code == -99999) e_code = 0;		/* -99999 for non-executing options like --version */
+    return (e_code);                /* exit the program */
 }
-else {
-    cpl_msg_warning(fn, "unloading of plugins is inhibited by program "
-                    "option 'unload-plugin'!");
-}
-
-p = cpl_parameterlist_find (caller_parameter_list, PACKAGE_RESOURCE ".mem-check");
-flag_mem_check = cpl_parameter_get_bool (p);
-
-
-/* cleanup allocated structures and memory */
-
-clean_up:
-cpl_parameterlist_delete (caller_parameter_list);
-er_stringarray_delete (set_of_frames_filenames);
-
-if (conf_file_local != NULL) 
-   {
-   cpl_free(conf_file_local);
-   }
-if (conf_file_global != NULL) 
-   {
-   cpl_free(conf_file_global);
-   }
-
-(void) er_help_free();
-(void) er_manage_sources(3," ",&conf_file_local);	/* free buffer of sources */
-
-
-if (flag_mem_check != 0)	/* see if we should check the memory */
-   {
-   cpl_memory_dump(); 
-   }
-
-end_of_it:
-cpl_end ();			/* stop subsystems of CPL */
-
-if (e_code == -99999) e_code = 0;		/* -99999 for non-executing options like --version */
-return (e_code);                /* exit the program */
-}
-
-
 /**@}*/
-
-
 /* End of file */
