@@ -1,4 +1,4 @@
-/* $Id: er_paf.c,v 1.18 2012/12/13 18:40:38 cgarcia Exp $
+/* $Id: er_paf.c,v 1.20 2013-08-23 15:04:22 cgarcia Exp $
  *
  *   This file is part of the ESO Common Pipeline Library
  *   Copyright (C) 2001-2004 European Southern Observatory
@@ -15,14 +15,14 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /*
  * $Author: cgarcia $
- * $Date: 2012/12/13 18:40:38 $
- * $Revision: 1.18 $
- * $Name: esorex-3_10 $
+ * $Date: 2013-08-23 15:04:22 $
+ * $Revision: 1.20 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -93,15 +93,6 @@ int er_create_recipe_pafs
 	return CPL_ERROR_BAD_FILE_FORMAT;
 	}
     
-    /* Read the ARCFILE keyword from the first raw file */
-    arcfile = er_paf_get_arcfile(frames);
-    if(arcfile == NULL)
-	{
-	cpl_msg_error(er_func, "Could not read ARCFILE from raw frames");
-	er_paf_config_delete(paf_configuration);
-	return CPL_ERROR_DATA_NOT_FOUND;
-	}
-
     /* Loop into all the products */
     cpl_msg_debug(__func__,"Looping into all the products");
     frame = cpl_frameset_get_first_const(frames);
@@ -143,6 +134,18 @@ int er_create_recipe_pafs
 		cpl_msg_debug(er_func, "Matched with regexp =%s", 
 			      config_item->pro_catg_regexp);
 		
+                /* Read the ARCFILE keyword from the first raw file */
+                if(arcfile == NULL)
+                    {
+                    arcfile = er_paf_get_arcfile(frames);
+                    if(arcfile == NULL)
+                        {
+                        cpl_msg_error(er_func, "Could not read ARCFILE from raw frames");
+                        er_paf_config_delete(paf_configuration);
+                        return CPL_ERROR_DATA_NOT_FOUND;
+                        }
+                    }
+
 		/* Decide what to do depending on number of extensions */
 		next = cpl_frame_get_nextensions(frame);
 		if(next == 0)
@@ -175,7 +178,8 @@ int er_create_recipe_pafs
 
     /* Cleanup */
     er_paf_config_delete(paf_configuration);
-    cpl_free(arcfile);
+    if(arcfile != NULL)
+        cpl_free(arcfile);
 
     return paf_err;
     }
@@ -217,7 +221,7 @@ int er_paf_single_hdu_fill_write
 	}
 
     /* Get the PAF filename */
-    paf_filename = cpl_sprintf("qc_%d.paf",idx_paf);
+    paf_filename = cpl_sprintf("qc_%04d.paf",idx_paf);
 
     /* Writing PAF file */
     cpl_dfs_save_paf(cpl_propertylist_get_string
@@ -284,7 +288,7 @@ int er_paf_multi_hdu_fill_write
 	    }
 	
 	/* Get the PAF filename */
-	paf_filename = cpl_sprintf("qc_%d_%d.paf",idx_paf, iext);
+	paf_filename = cpl_sprintf("qc_%04d_%04d.paf",idx_paf, iext);
 
 	/* Writing PAF file */
 	cpl_dfs_save_paf(cpl_propertylist_get_string
