@@ -1,4 +1,4 @@
-/* $Id: er_plugin.c,v 1.115 2013/02/19 15:54:36 rpalsa Exp $
+/* $Id: er_plugin.c,v 1.115 2013-02-19 15:54:36 rpalsa Exp $
  *
  *   This file is part of the ESO Common Pipeline Library
  *   Copyright (C) 2001-2012 European Southern Observatory
@@ -15,14 +15,14 @@
  *
  *   You should have received a copy of the GNU General Public License
  *   along with this program; if not, write to the Free Software
- *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /*
  * $Author: rpalsa $
- * $Date: 2013/02/19 15:54:36 $
+ * $Date: 2013-02-19 15:54:36 $
  * $Revision: 1.115 $
- * $Name: esorex-3_10 $
+ * $Name: not supported by cvs2svn $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -936,7 +936,6 @@ int move_products (cpl_frameset * frameset,
 int plugin_process_plugin (cpl_parameterlist *caller_parameters,
             char *plugin_name,
             er_stringarray_t * sof_filename_list, 
-            char *date_str,
             int argc, char * argv[])
 
 {
@@ -1049,7 +1048,7 @@ if (log_err != CPL_ERROR_NONE)
 cpl_msg_set_level (CPL_MSG_OFF);
 cdescr = cpl_get_description(CPL_DESCRIPTION_DEFAULT);
 
-(void) cpl_msg_info(er_func,"Esorex, version %s, as of %s",PACKAGE_VERSION,date_str);
+(void) cpl_msg_info(er_func,"Esorex, version %s", PACKAGE_VERSION);
 /*
 */
 (void) cpl_msg_info(er_func,"using the libraries: %s\n",cdescr);
@@ -1174,7 +1173,8 @@ if (flag_plugin_found != 0)			/*  = TRUE */
 
    cpp = cpl_plugin_get_name ((cpl_plugin *) trecipe);
 
-#ifdef USE_GLOBAL_PLUGIN_RC /* Parse any global plugin configuration file */
+
+   /* Parse any global plugin configuration file */
 
    cptr = getenv ("HOME");
    if (cptr == NULL)
@@ -1218,41 +1218,6 @@ if (flag_plugin_found != 0)			/*  = TRUE */
          if (e_code != 0) goto plugin_deinit;
          }
       }
-#endif
-
-#ifdef USE_LOCAL_PLUGIN_RC /* Parse any local plugin configuration file */
-
-   if (cpp != NULL) 
-      n = (int)strlen(cpp);
-   else
-      n = 0;
-   n = n + ((int) strlen(LOCAL_RC_PATH)) + ((int) strlen(GLOBAL_RC_EXTENSION));
-   n += 4;                                 /* for the additionial const chars */
-
-   plugin_conf_file_local = (char *) cpl_malloc((size_t) n);
-   if ( plugin_conf_file_local == NULL)
-      {
-      cpl_msg_error (er_func,
-                     "Could not allocate %d bytes for plugin_conf_file_local",n);
-      e_code = CPL_ERROR_ILLEGAL_OUTPUT;
-      goto plugin_deinit;
-      }
-
-   void) strcpy(plugin_conf_file_local,LOCAL_RC_PATH);
-   (void) strcat(plugin_conf_file_local, "/");
-   if (cpp != NULL) (void) strcat(plugin_conf_file_local, cpp);
-   (void) strcat(plugin_conf_file_local, GLOBAL_RC_EXTENSION);
-
-   if (e_code == 0)
-      {
-      if (fileutils_file_exists(plugin_conf_file_local) != 0)
-         {
-         e_code = params_parse_config_file (trecipe->parameters,
-                                            plugin_conf_file_local);
-         if (e_code != 0) goto plugin_deinit;
-         }
-      }
-#endif
 
    /* Parse any specifically specified plugin configuration file */
    if (e_code == 0)
